@@ -1,5 +1,5 @@
 /*
- *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2013
+ *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2020
  *	<manutm007@gmail.com>
  *
  * 	This program is free software: you can redistribute it and/or modify
@@ -31,7 +31,7 @@ static void new_version_available_win(char *version_num)
 			NULL);
 
 	close_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL_CLOSE);
-	close_but = close_but;	/* To get rid of one annoying compiler warning */
+	close_but = close_but;
 
 	set_tickr_icon_to_dialog(GTK_WINDOW(dialog));
 	gtk_window_set_position(GTK_WINDOW(dialog), GTK_WIN_POS_CENTER);
@@ -45,9 +45,6 @@ static void new_version_available_win(char *version_num)
 	gtk_container_set_border_width(GTK_CONTAINER(table), 15);
 	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);
 
-	/* Also send to stdout */
-	fprintf(STD_OUT, "%s version %s is available for download from: <%s>\n", APP_NAME,
-		version_num, DOWNLOAD_URL);
 	snprintf(str1, 256, "%s version <b>%s</b> is available for download from:", APP_NAME,
 		version_num);
 	snprintf(str2, 256, "<a href=\"%s\">%s</a>", DOWNLOAD_URL, DOWNLOAD_URL);
@@ -71,28 +68,33 @@ void check_for_updates()
 {
 	sockt	sock;
 	char	*response, *new_url;
-	int	/*status,*/ recv_status;
-	char	lsvn[32], tmp[32], *p = tmp;
+	int	recv_status;
+	char	last_stable_version[32], installed_version[32], *p;
+	int	id_str_len, resp_len;
 	int	info = OK + 1;
 
-	fprintf(STD_OUT, "Checking for updates:\n");
+	INFO_OUT("Checking for updates:\n")
+
 	if (connect_with_url(&sock, CHECK4UPDATES_URL) == OK) {
-		/* TODO: check this (when http 'connection closed by server' is returned):
-		if ((status = get_http_response(sock, "GET", "", CHECK4UPDATES_URL, &new_url,\
-				&response, &recv_status)) == SEND_ERROR || status == RECV_ERROR) {
-			if (recv_status == CONNECTION_CLOSED_BY_SERVER || recv_status == SOCK_SHOULD_BE_CLOSED) {
-				CLOSE_SOCK(sock);
-				if (connect_with_url(&sock, CHECK4UPDATES_URL) == OK)
-					status = get_http_response(sock, "GET", "", CHECK4UPDATES_URL, &new_url,\
-						&response, &recv_status);
-			}
-		}
-		if (status == OK) {*/
 		if (get_http_response(sock, "GET", "", CHECK4UPDATES_URL, &new_url, &response, &recv_status) == OK) {
-			remove_chunk_info(&response);
-			if (strncmp(response, CHECK4UPDATES_ID_STR, strlen(CHECK4UPDATES_ID_STR)) == 0) {
-				str_n_cpy(lsvn, response + strlen(CHECK4UPDATES_ID_STR), 31);
-				str_n_cpy(p, APP_VERSION_NUMBER, 31);
+			id_str_len = strlen(CHECK4UPDATES_ID_STR);
+			resp_len = strlen(response);
+			p = response;
+			last_stable_version[0] = '\0';
+
+			while (p < response + resp_len - id_str_len) {
+				if (strncmp(p, CHECK4UPDATES_ID_STR, id_str_len) == 0) {
+					str_n_cpy(last_stable_version, p + id_str_len, 31);
+					remove_trailing_whitespaces_from_str(last_stable_version);
+					break;
+				} else
+					p++;
+			}
+
+			if (last_stable_version[0] != '\0') {
+				p = installed_version;
+				str_n_cpy(p, APP_V_NUM, 31);
+
 				while (*p != '\0') {
 					p++;
 					if (*p == '~') {
@@ -102,14 +104,16 @@ void check_for_updates()
 						break;
 					}
 				}
-				if (strcmp(lsvn, tmp) > 0)
-					new_version_available_win(lsvn);
-				else {
-					/* also send to stdout */
-					fprintf(STD_OUT, APP_NAME " is up to date  :)\n");
+
+				if (strcmp(last_stable_version, installed_version) > 0) {
+					new_version_available_win(last_stable_version);
+					INFO_OUT("%s version %s is available for download from: <%s>\n",
+						APP_NAME, last_stable_version, DOWNLOAD_URL)
+				} else {
 					info_win("Check for Updates",
 						"\n            " APP_NAME " is up to date  :)            \n",
 						INFO, FALSE);
+					INFO_OUT(APP_NAME " is up to date  :)\n")
 				}
 				info = OK;
 			}
@@ -118,5 +122,5 @@ void check_for_updates()
 		CLOSE_SOCK(sock);
 	}
 	if (info != OK)
-		warning(FALSE, 1, "Couldn't retrieve requested information from website");
+		warning(BLOCK, "Couldn't retrieve requested information from website");
 }
