@@ -1,5 +1,5 @@
 /*
- *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2013
+ *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2020
  *	<manutm007@gmail.com>
  *
  * 	This program is free software: you can redistribute it and/or modify
@@ -21,25 +21,19 @@
 #define URL_ENTRY_LENGTH	100
 #define RANK_ENTRY_LENGTH	FLIST_RANK_STR_LEN
 
-/* Predefined GTK_RESPONSE_ are < 0, those ones are app level defined and >= 0. */
-enum {
-	GTK_RESPONSE_SELECT_ALL, GTK_RESPONSE_UNSELECT_ALL, GTK_RESPONSE_CLEAR_RANKING, GTK_RESPONSE_TOP,
-	GTK_RESPONSE_HOME, GTK_RESPONSE_REMOVE, GTK_RESPONSE_ADD, GTK_RESPONSE_SINGLE, GTK_RESPONSE_SELECTION
-};
-
 enum {COLUMN_INT, COLUMN_BOOLEAN_CHECKED, COLUMN_STRING_RANK, COLUMN_STRING_TITLE, COLUMN_STRING_URL, N_COLUMNS};
 
 static GtkWidget	*dialog, *url_entry, *rank_entry;
 static FList		*flist;
 static int		f_index;	/* Starting at 0 (row starts at 1) */
-static gchar		home_feed[FILE_NAME_MAXLEN + 1];
+static char		home_feed[FILE_NAME_MAXLEN + 1];
 
-/* Tree selection callback - update f_index and copy selected url to url_entry (override) */
-static gint tree_selection_changed(GtkTreeSelection *selection)
+/* Tree selection callback - update f_index and copy selected URL to url_entry (override) */
+static int tree_selection_changed(GtkTreeSelection *selection)
 {
 	GtkTreeModel	*tree_model = NULL;
 	GtkTreeIter	iter;
-	gchar		*url_str, *rank_str;
+	char		*url_str, *rank_str;
 
 	if (gtk_tree_selection_get_selected(selection, &tree_model, &iter)) {
 		gtk_tree_model_get(tree_model, &iter,
@@ -60,11 +54,11 @@ static gint tree_selection_changed(GtkTreeSelection *selection)
 }
 
 /* Catch double-click on tree view */
-static gint double_click_on_tree_view(GtkTreeView *tree_view, GtkTreePath *tree_path)
+static int double_click_on_tree_view(GtkTreeView *tree_view, GtkTreePath *tree_path)
 {
 	GtkTreeModel	*tree_model = NULL;
 	GtkTreeIter	iter;
-	gchar		*url_str, *rank_str;
+	char		*url_str, *rank_str;
 
 	tree_model = gtk_tree_view_get_model(tree_view);
 	if (gtk_tree_model_get_iter(tree_model, &iter, tree_path)) {
@@ -86,14 +80,14 @@ static gint double_click_on_tree_view(GtkTreeView *tree_view, GtkTreePath *tree_
 	return TRUE;
 }
 
-static gint enter_key_pressed_in_entry(GtkWidget *widget)
+static int enter_key_pressed_in_entry(GtkWidget *widget)
 {
 	widget = widget;
-	gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_ADD);
+	gtk_dialog_response(GTK_DIALOG(dialog), GTK_RESPONSE_ADD_UPD);
 	return TRUE;
 }
 
-static gint clear_entry(GtkWidget *widget)
+static int clear_entry(GtkWidget *widget)
 {
 	widget = widget;
 	gtk_entry_set_text(GTK_ENTRY(url_entry), "");
@@ -103,7 +97,7 @@ static gint clear_entry(GtkWidget *widget)
 	return TRUE;
 }
 
-static gint checkbox_toggled(GtkCellRendererToggle *renderer, gchar *path_str, gpointer tree_view)
+static int checkbox_toggled(GtkCellRendererToggle *renderer, char *path_str, gpointer tree_view)
 {
 	GtkTreeModel	*tree_model;
 	GtkTreeIter	iter;
@@ -233,6 +227,7 @@ static void fill_list_store_from_flnode(GtkListStore *list_store)
 		}
 }
 
+/* Use url_entry and rank_entry (global vars in this src file) */
 static int add_feed_to_flnode_and_list_store(Resource *resrc, GtkListStore *list_store)
 {
 	FList	*flist2;
@@ -242,50 +237,54 @@ static int add_feed_to_flnode_and_list_store(Resource *resrc, GtkListStore *list
 	char	rss_feed_ttl[32];
 	char	rank[FLIST_RANK_STR_LEN + 1];
 	int	unused;
+	char	resrc_id_bak[FILE_NAME_MAXLEN + 1];
 
+	str_n_cpy(resrc_id_bak, resrc->id, FILE_NAME_MAXLEN);
 	str_n_cpy(resrc->id, (char *)gtk_entry_get_text(GTK_ENTRY(url_entry)), FILE_NAME_MAXLEN);
-	if (get_params()->enable_feed_ordering != 'y')
-		if ((flist2 = f_list_nth(flist, f_list_search(flist, resrc->id) + 1)) != NULL)
-			str_n_cpy(rank, flist2->rank, FLIST_RANK_STR_LEN);
-		else
-			str_n_cpy(rank, BLANK_STR_16, FLIST_RANK_STR_LEN);
-	else
-		str_n_cpy(rank, (char *)gtk_entry_get_text(GTK_ENTRY(rank_entry)), FLIST_RANK_STR_LEN);
 	if ((rss_status = get_feed(resrc, get_params())) == OK) {
+		if (get_params()->enable_feed_ordering != 'y') {
+			if (IS_FLIST(flist) && (flist2 = f_list_nth(flist, f_list_search(flist, resrc->id) + 1)) != NULL)
+				str_n_cpy(rank, flist2->rank, FLIST_RANK_STR_LEN);
+			else
+				str_n_cpy(rank, BLANK_STR_16, FLIST_RANK_STR_LEN);
+		} else
+			str_n_cpy(rank, (char *)gtk_entry_get_text(GTK_ENTRY(rank_entry)), FLIST_RANK_STR_LEN);
+		/* Fix LP bug #1272129 */
 		if (IS_FLIST(flist))
 			flist = f_list_last(flist);
+		/* If valid, added URL is set selected */
 		if (get_feed_info(&unused, resrc->id, rss_feed_title, rss_feed_link, rss_feed_ttl) == OK)
-			/* Added url is set unselected. Is that OK ? */
-			flist = f_list_add_at_end(flist, resrc->id, rss_feed_title, FALSE, rank);
+			flist = f_list_add_at_end(flist, resrc->id, rss_feed_title, rss_status == OK ? TRUE : FALSE, rank);
 		else
-			flist = f_list_add_at_end(flist, resrc->id, "", FALSE, rank);
+			flist = f_list_add_at_end(flist, resrc->id, "", rss_status == OK ? TRUE : FALSE, rank);
 		flist = f_list_sort(flist);
 		fill_list_store_from_flnode(list_store);
-	} /*else {
-		// Should something be done here ?
-		if (question_win("URL is unreachable. Keep it anyways ?") == YES) {
-		}
-	}*/
+	} else {
+		/* === TODO: Find out how to handle an invalid or unreachable URL === */
+		warning(BLOCK, "URL is invalid or unreachable (or whatever)");
+		return RESOURCE_INVALID;
+	}
 	return rss_status;
 }
 
 /*
- * Open a dialog with a list of urls to choose from and an url entry.
+ * Open a dialog with a list of URLs to choose from and an URL entry.
  *
- * Entry format in url list file:
- *	['*' (selected) or '-' (unselected) + "000" (3 chars rank) + url [+ '>' + title] + '\n']
+ * Entry format in URL list file:
+ *	['*' (selected) or '-' (unselected) + "000" (3 chars rank) + URL [+ '>' + title] + '\n']
  *
  * Entry max length = FILE_NAME_MAXLEN
  * See also:	(UN)SELECTED_URL_CHAR/STR and TITLE_TAG_CHAR/STR in tickr.h
  * 		FLIST_RANK_FORMAT and FLIST_RANK_STR_LEN in tickr_list.h
  */
-void get_new_url(Resource *resrc)
+void manage_list_and_selection(Resource *resrc)
 {
 	TickerEnv		*env;
 	GtkWidget		*sc_win, *hbox;
-	GtkWidget		*cancel_but, *selectall_but, *unselectall_but;
-	GtkWidget		*clear_ranking_but, *top_but, *home_but, *remove_but;
-	GtkWidget		*add_but, *single_but, *selection_but, *clear_but;
+	GtkWidget		*cancel_but, *selectall_but, *unselectall_but, *clear_ranking_but;
+	GtkWidget		*top_but, *current_but, *home_but, *remove_but, *add_but;
+	GtkWidget		*single_but, *selection_but, *clear_but;
+	GtkWidget		*tmp_label;
 	GtkTreeView		*tree_view = NULL;
 	GtkListStore		*list_store = NULL;
 	GtkTreeModel		*tree_model = NULL;
@@ -294,16 +293,17 @@ void get_new_url(Resource *resrc)
 	GtkTreeViewColumn	*column1 = NULL, *column2 = NULL, *column3 = NULL, *column4 = NULL;
 	GtkTreeSelection	*selection = NULL;
 	FList			*flist_bak = NULL, *flist2 = NULL;
-	Resource		resrc_bak;
-	gint			response;
-	gint			cursor_position;
+	char			resrc_id_bak[FILE_NAME_MAXLEN + 1];
+	int			response;
+	int			cursor_position;
+	char			*tooltip_text;
 
 	env = get_ticker_env();
 	env->suspend_rq = TRUE;
 	gtk_window_set_keep_above(GTK_WINDOW(env->win), FALSE);
 
 	dialog = gtk_dialog_new_with_buttons(
-			"Feed Picker (RSS/Atom)", GTK_WINDOW(env->win),
+			"Feed Organizer (RSS/Atom)", GTK_WINDOW(env->win),
 			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 			NULL);
 
@@ -313,18 +313,32 @@ void get_new_url(Resource *resrc)
 	if (get_params()->enable_feed_ordering == 'y')
 		clear_ranking_but = gtk_dialog_add_button(GTK_DIALOG(dialog), "Clear Rank", GTK_RESPONSE_CLEAR_RANKING);
 	top_but = gtk_dialog_add_button(GTK_DIALOG(dialog), "Top", GTK_RESPONSE_TOP);
+	current_but = gtk_dialog_add_button(GTK_DIALOG(dialog), "Current", GTK_RESPONSE_CURRENT);
 	home_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_HOME, GTK_RESPONSE_HOME);
 	remove_but = gtk_dialog_add_button(GTK_DIALOG(dialog), GTK_STOCK_REMOVE, GTK_RESPONSE_REMOVE);
-	add_but = gtk_dialog_add_button(GTK_DIALOG(dialog), "Add/Upd", GTK_RESPONSE_ADD);
+	add_but = gtk_dialog_add_button(GTK_DIALOG(dialog), "Add/Upd", GTK_RESPONSE_ADD_UPD);
 	single_but = gtk_dialog_add_button(GTK_DIALOG(dialog), "OK (Single)", GTK_RESPONSE_SINGLE);
 	selection_but = gtk_dialog_add_button(GTK_DIALOG(dialog), "OK (Selec)", GTK_RESPONSE_SELECTION);
 	clear_but = gtk_button_new_with_label("Clear");
 
+	/* Tooltips */
+	tooltip_text = l_str_new("Will try to connect and add or update URL and title");
+	gtk_widget_set_tooltip_text(add_but, tooltip_text);
+	l_str_free(tooltip_text);
+	tooltip_text = l_str_new("'Single selection' mode - Will read only the highlighted URL");
+	gtk_widget_set_tooltip_text(single_but, tooltip_text);
+	l_str_free(tooltip_text);
+	tooltip_text = l_str_new("'Multiple selection' mode - Will read sequentially all selected URLs, "
+		"starting from highlighted one - more exactly URL in entry (if any) / first one otherwise");
+	gtk_widget_set_tooltip_text(selection_but, tooltip_text);
+	l_str_free(tooltip_text);
+
 	cancel_but = cancel_but;	/* To get rid of some annoying compiler warnings */
 	selectall_but = selectall_but;
 	unselectall_but = unselectall_but;
 	clear_ranking_but = clear_ranking_but;
 	top_but = top_but;
+	current_but = current_but;
 	home_but = home_but;
 	remove_but = remove_but;
 	add_but = add_but;
@@ -341,7 +355,7 @@ void get_new_url(Resource *resrc)
 	gtk_container_set_border_width(GTK_CONTAINER(sc_win), 5);
 	/* Whole window must be visible on netbooks as well */
 	gtk_widget_set_size_request(sc_win, 1000, 450);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), sc_win, TRUE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), sc_win, TRUE, TRUE, 0);
 
 	str_n_cpy(home_feed, get_params()->homefeed, FILE_NAME_MAXLEN);
 	list_store = gtk_list_store_new(N_COLUMNS, G_TYPE_INT, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
@@ -367,7 +381,7 @@ void get_new_url(Resource *resrc)
 	if (get_params()->enable_feed_ordering == 'y') {
 		renderer2 = gtk_cell_renderer_text_new();
 #ifndef G_OS_WIN32
-		/* Doesn't compile on win32 because of older GTK version. */
+		/* Doesn't compile on win32 because of older GTK version */
 		gtk_cell_renderer_set_alignment(renderer2, 1.0, 0.5);
 #endif
 		column2 = gtk_tree_view_column_new_with_attributes(NULL/*"Rank"*/, renderer2,
@@ -389,16 +403,17 @@ void get_new_url(Resource *resrc)
 
 	selection = gtk_tree_view_get_selection(tree_view);
 	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);
-
-	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(tree_selection_changed), NULL);
+	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(tree_selection_changed), selection);
 	g_signal_connect(G_OBJECT(tree_view), "row_activated", G_CALLBACK(double_click_on_tree_view), NULL);
 	g_signal_connect(G_OBJECT(renderer1), "toggled", G_CALLBACK(checkbox_toggled), tree_view);
 
 	hbox = gtk_hbox_new(FALSE, 0);
 	if (get_params()->enable_feed_ordering != 'y')
-		gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Enter new URL:"), FALSE, FALSE, 0);
+		tmp_label = gtk_label_new("<b>New Feed</b>  ->  Enter URL:");	/* or <span foreground=\"red\"></span>, <big></big>, ... */
 	else
-		gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("Enter rank and new URL:"), FALSE, FALSE, 0);
+		tmp_label = gtk_label_new("<b>New Feed</b>  ->  Enter rank and URL:");
+	gtk_label_set_use_markup(GTK_LABEL(tmp_label), TRUE);
+	gtk_box_pack_start(GTK_BOX(hbox), tmp_label, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new("  "), TRUE, FALSE, 0);
 	if (get_params()->enable_feed_ordering == 'y') {
 		rank_entry = gtk_entry_new();
@@ -411,7 +426,7 @@ void get_new_url(Resource *resrc)
 	if (get_params()->enable_feed_ordering != 'y')
 		gtk_entry_set_width_chars(GTK_ENTRY(url_entry), URL_ENTRY_LENGTH);
 	else
-		gtk_entry_set_width_chars(GTK_ENTRY(url_entry), URL_ENTRY_LENGTH - 10);	/* about... */
+		gtk_entry_set_width_chars(GTK_ENTRY(url_entry), URL_ENTRY_LENGTH - 10);	/* about ... */
 	gtk_box_pack_start(GTK_BOX(hbox), url_entry, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), clear_but, FALSE, FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
@@ -427,27 +442,40 @@ void get_new_url(Resource *resrc)
 	gtk_widget_show_all(dialog);
 
 	/* Backup last valid opened resource (if any) */
-	str_n_cpy(resrc_bak.id, resrc->id, FILE_NAME_MAXLEN);
-	/* Is it better like that ? */
-	if (IS_FLIST(flist) && resrc->id[0] != '\0')
+	str_n_cpy(resrc_id_bak, resrc->id, FILE_NAME_MAXLEN);
+	if (IS_FLIST(flist) && resrc->id[0] != '\0') {
 		if ((f_index = f_list_search(flist, resrc->id)) > -1)
 			highlight_and_go_to_row(tree_view, selection, f_index + 1);
+		/*
+		 * Try original URL in case of HTTP redirects
+		 * TODO: Find best practice - should new URL replace original URL ?
+		 */
+		else if ((f_index = f_list_search(flist, resrc->orig_url)) > -1)
+			highlight_and_go_to_row(tree_view, selection, f_index + 1);
+	}
 	gtk_widget_grab_focus(GTK_WIDGET(tree_view));
 
 	while ((response = gtk_dialog_run(GTK_DIALOG(dialog))) != GTK_RESPONSE_CANCEL_CLOSE) {
 		if (response == GTK_RESPONSE_SELECT_ALL) {
-			if (question_win("Select all URLs ?") == YES)
+			if (question_win("Select all URLs ?", NO) == YES)
 				select_all(tree_view);
 		} else if (response == GTK_RESPONSE_UNSELECT_ALL) {
-			if (question_win("Unselect all URLs ?") == YES)
+			if (question_win("Unselect all URLs ?", NO) == YES)
 				unselect_all(tree_view);
 		} else if (response == GTK_RESPONSE_CLEAR_RANKING) {
-			if (question_win("Clear all URLs ranking ?") == YES)
+			if (question_win("Clear all URLs ranking ?", NO) == YES)
 				clear_ranking(tree_view);
 		} else if (response == GTK_RESPONSE_TOP) {
 			highlight_and_go_to_row(tree_view, selection, 1);
+		} else if (response == GTK_RESPONSE_CURRENT) {
+			gtk_entry_set_text(GTK_ENTRY(url_entry), (const char *)resrc->id);
+			if (IS_FLIST(flist)) {
+				if ((f_index = f_list_search(flist,
+						gtk_entry_get_text(GTK_ENTRY(url_entry)))) > -1)
+					highlight_and_go_to_row(tree_view, selection, f_index + 1);
+			}
 		} else if (response == GTK_RESPONSE_HOME) {
-			gtk_entry_set_text(GTK_ENTRY(url_entry), (const gchar *)home_feed);
+			gtk_entry_set_text(GTK_ENTRY(url_entry), (const char *)home_feed);
 			if (IS_FLIST(flist)) {
 				if ((f_index = f_list_search(flist,
 						gtk_entry_get_text(GTK_ENTRY(url_entry)))) > -1)
@@ -456,7 +484,7 @@ void get_new_url(Resource *resrc)
 		} else if (response == GTK_RESPONSE_REMOVE) {
 			if (IS_FLIST(flist) && f_index > -1) {
 				if (gtk_tree_selection_get_selected(selection, &tree_model, &iter) &&
-						question_win("Remove selected URL from list ?") == YES) {
+						question_win("Remove highlighted URL from list ?", NO) == YES) {
 					gtk_list_store_remove(list_store, &iter);
 					flist = f_list_remove(flist);
 					if (f_list_count(flist) > 0)
@@ -467,18 +495,18 @@ void get_new_url(Resource *resrc)
 					gtk_entry_set_text(GTK_ENTRY(url_entry), "");
 				}
 			} else
-				warning(FALSE, 1, "You must select an URL first");
-		} else if (response == GTK_RESPONSE_ADD) {	/* Actually add/update */
+				warning(BLOCK, "You must select an URL first");
+		} else if (response == GTK_RESPONSE_ADD_UPD) {
 			if (get_params()->enable_feed_ordering == 'y') {
-				if (!(str_is_num(gtk_entry_get_text(GTK_ENTRY(rank_entry))) &&\
-						atoi(gtk_entry_get_text(GTK_ENTRY(rank_entry))) >= 0) &&\
+				if (!(str_is_num(gtk_entry_get_text(GTK_ENTRY(rank_entry))) &&
+						atoi(gtk_entry_get_text(GTK_ENTRY(rank_entry))) >= 0) &&
 						!str_is_blank(gtk_entry_get_text(GTK_ENTRY(rank_entry)))) {
-					warning(FALSE, 1, "You must enter a numerical (>= 0 integer) value");
+					warning(BLOCK, "You must enter a numerical (>= 0 integer) value");
 					gtk_widget_grab_focus(GTK_WIDGET(rank_entry));
 					continue;
 				}
 			}
-			if (gtk_entry_get_text(GTK_ENTRY(url_entry))[0] != '\0') {
+			if (gtk_entry_get_text(GTK_ENTRY(url_entry))[0] != '\0') {	// FIXME: Some issues here
 				cursor_position = gtk_editable_get_position(GTK_EDITABLE(url_entry));
 				str_n_cpy(resrc->id, gtk_entry_get_text(GTK_ENTRY(url_entry)), FILE_NAME_MAXLEN);
 				if (add_feed_to_flnode_and_list_store(resrc, list_store) == OK) {
@@ -486,16 +514,18 @@ void get_new_url(Resource *resrc)
 							gtk_entry_get_text(GTK_ENTRY(url_entry)))) > -1)
 						highlight_and_go_to_row(tree_view, selection, f_index + 1);
 					/* Backup last valid opened resource (if any) */
-					str_n_cpy(resrc_bak.id, resrc->id, FILE_NAME_MAXLEN);
+					str_n_cpy(resrc_id_bak, resrc->id, FILE_NAME_MAXLEN);
 				} else {
+					/* === TODO: Find out how to handle an invalid or unreachable URL === */
 					/* Restore last valid opened resource (if any) */
-					str_n_cpy(resrc->id, resrc_bak.id, FILE_NAME_MAXLEN);
+					/*str_n_cpy(resrc->id, resrc_id_bak, FILE_NAME_MAXLEN);
 					gtk_widget_grab_focus(GTK_WIDGET(url_entry));
 					gtk_editable_set_position(GTK_EDITABLE(url_entry), cursor_position);
-					continue;
+					continue;*/
+					break;
 				}
 			} else {
-				warning(FALSE, 1, "You must enter an URL first");
+				warning(BLOCK, "You must enter an URL first");
 				gtk_widget_grab_focus(GTK_WIDGET(url_entry));
 				continue;
 			}
@@ -507,11 +537,14 @@ void get_new_url(Resource *resrc)
 					env->selection_mode = SINGLE;
 					env->reload_rq = TRUE;
 					break;
-				} else
+				} else {
+					/* === TODO: Find out how to handle an invalid or unreachable URL === */
 					/* Restore last valid opened resource (if any) */
-					str_n_cpy(resrc->id, resrc_bak.id, FILE_NAME_MAXLEN);
+					/*str_n_cpy(resrc->id, resrc_id_bak, FILE_NAME_MAXLEN);*/
+					break;
+				}
 			} else
-				warning(FALSE, 1, "You must enter or select an URL first");
+				warning(BLOCK, "You must enter or select an URL first");
 		} else if (response == GTK_RESPONSE_SELECTION) {
 			if (IS_FLIST(flist)) {
 				for (flist2 = f_list_first(flist); IS_FLIST(flist2); flist2 = flist2->next)
@@ -524,9 +557,9 @@ void get_new_url(Resource *resrc)
 						break;
 					}
 				}
-				warning(FALSE, 1, "Selection is empty");
+				warning(BLOCK, "Selection is empty");
 			} else
-				warning(FALSE, 1, "URL list is empty");
+				warning(BLOCK, "URL list is empty");
 		}
 		gtk_widget_grab_focus(GTK_WIDGET(tree_view));
 	}
@@ -540,8 +573,8 @@ void get_new_url(Resource *resrc)
 			build_feed_selection_from_feed_list();
 			if (IS_FLIST(get_feed_selection()) && response == GTK_RESPONSE_SELECTION) {
 				/*
-				 * NEW: Multiple selection mode: start reading selection with highlighted feed
-				 * - more exactly url in entry (if any) / first one otherwise.
+				 * Multiple selection mode: start reading selection with highlighted feed
+				 * - more exactly URL in entry (if any) / first one otherwise
 				 */
 				if ((f_index = f_list_search(get_feed_selection(), gtk_entry_get_text(GTK_ENTRY(url_entry)))) > -1)
 					set_feed_selection(f_list_nth(get_feed_selection(), f_index + 1));
@@ -551,12 +584,13 @@ void get_new_url(Resource *resrc)
 			if (IS_FLIST(flist_bak))
 				f_list_free_all(flist_bak);
 		} else {
-			info_win("", "Invalid new feed list", INFO_ERROR, TRUE);
-			fprintf(STD_ERR, "Invalid new feed list\n");
+			/*info_win("", "Invalid new feed list", INFO_ERROR, TRUE);
+			fprintf(STD_ERR, "Invalid new feed list\n");*/
+			warning(BLOCK, "Invalid new feed list"),
 			flist = flist_bak;
 		}
 	} else {
-		/* If cancelled, we only restore feed list. */
+		/* If cancelled, we only restore feed list */
 		if (IS_FLIST(flist))
 			f_list_free_all(flist);
 		flist = flist_bak;
