@@ -1,5 +1,5 @@
 /*
- *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2013
+ *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2020
  *	<manutm007@gmail.com>
  *
  * 	This program is free software: you can redistribute it and/or modify
@@ -21,10 +21,10 @@
 
 #define FLIST_URL_MAXLEN	FILE_NAME_MAXLEN
 #define FLIST_TITLE_MAXLEN	FEED_TITLE_MAXLEN
-#define N_FLIST_MAX		(32 * 1024)	/* To prevent infinite loops - adjust as necessary */
-#define FLIST_SIG		"fl_node"	/* Make sure FLIST_SIG_LEN = strlen(FLIST_SIG) */
+#define N_FLIST_MAX		(32 * 1024)		/* To prevent infinite loops - adjust as necessary */
+#define FLIST_SIG		"fl_node"		/* Make sure FLIST_SIG_LEN = strlen(FLIST_SIG) */
 #define FLIST_SIG_LEN		7
-#define FLIST_RANK_FORMAT	"%3d"		/* Make sure format and length match */
+#define FLIST_RANK_FORMAT	"%3d"			/* Make sure format and length match */
 #define FLIST_RANK_STR_LEN	3
 #define BLANK_STR_16		"                "	/* Make sure FLIST_RANK_STR_LEN < (BLANK_STR_)16 */
 
@@ -35,39 +35,38 @@
 	(node != NULL && strcmp(node->sig, FLIST_SIG) == 0)
 
 #define CHECK_IS_FLIST(node, func)\
-	if (!IS_FLIST(node)) {\
-		big_error(FLIST_ERROR, 2, func, ": Invalid node in list");\
-	}
+	if (!IS_FLIST(node))\
+		big_error(FLIST_ERROR, "%s(): Invalid node in list", func);
 
 typedef struct FeedListNode {
 	char			*url;
 	char			*title;
-	gboolean		selected;
+	zboolean		selected;
 	char			rank[FLIST_RANK_STR_LEN + 1];
 	struct FeedListNode	*prev;
 	struct FeedListNode	*next;
 	char			sig[FLIST_SIG_LEN + 1];
 } FList;
 
-FList	*f_list_new(const char *, const char *, gboolean, const char *);
-void	f_list_free(FList *);
-void	f_list_free_all(FList *);
-FList	*f_list_first(FList *);
-FList	*f_list_last(FList *);
-FList	*f_list_prev(FList *);
-FList	*f_list_next(FList *);
-FList	*f_list_nth(FList *, int);
-int	f_list_index(FList *);
-int	f_list_count(FList *);
-FList	*f_list_add_at_start(FList *, const char *, const char *, gboolean, const char *);
-FList	*f_list_add_at_end(FList *, const char *, const char *, gboolean, const char *);
-FList	*f_list_insert_before(FList *, const char *, const char *, gboolean, const char *);
-FList	*f_list_insert_after(FList *, const char *, const char *, gboolean, const char *);
-FList	*f_list_remove(FList *);
-void	f_list_swap(FList *, FList *);
-FList	*f_list_sort(FList *);
-int	f_list_search(FList *, const char *);
-int	f_list_load_from_file(FList **, const char *);
-int	f_list_save_to_file(FList *, const char *);
-FList	*f_list_clone(FList *);
+FList		*f_list_new(const char *, const char *, zboolean, const char *);
+void		f_list_free(FList *);
+void		f_list_free_all(FList *);
+FList		*f_list_first(FList *);
+FList		*f_list_last(FList *);
+FList		*f_list_prev(FList *);
+FList		*f_list_next(FList *);
+FList		*f_list_nth(FList *, int);
+int		f_list_index(FList *);
+int		f_list_count(FList *);
+FList		*f_list_add_at_start(FList *, const char *, const char *, zboolean, const char *);
+FList		*f_list_add_at_end(FList *, const char *, const char *, zboolean, const char *);
+FList		*f_list_insert_before(FList *, const char *, const char *, zboolean, const char *);
+FList		*f_list_insert_after(FList *, const char *, const char *, zboolean, const char *);
+FList		*f_list_remove(FList *);
+void		f_list_swap(FList *, FList *);
+FList		*f_list_sort(FList *);
+int		f_list_search(FList *, const char *);
+int		f_list_load_from_file(FList **, const char *);
+int		f_list_save_to_file(FList *, const char *);
+FList		*f_list_clone(FList *);
 #endif /* INC_TICKR_LIST_H */
