@@ -1,5 +1,5 @@
 /*
- *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2013
+ *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2020
  *	<manutm007@gmail.com>
  *
  * 	This program is free software: you can redistribute it and/or modify
@@ -28,7 +28,7 @@ void import_params()
 
 	gtk_window_set_keep_above(GTK_WINDOW(get_ticker_env()->win), FALSE);
 
-	dialog = gtk_file_chooser_dialog_new("Import Preferences", GTK_WINDOW(get_ticker_env()->win),
+	dialog = gtk_file_chooser_dialog_new("Import Settings", GTK_WINDOW(get_ticker_env()->win),
 			GTK_FILE_CHOOSER_ACTION_OPEN,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL_CLOSE,
 			GTK_STOCK_OK, GTK_RESPONSE_OK,
@@ -46,9 +46,9 @@ void import_params()
 		if ((file_name = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog))) != NULL) {
 			if ((fp1 = g_fopen(file_name, "rb")) != NULL) {
 				snprintf(tmp, TMPSTR_SIZE + 1,
-					"Preferences are about to be imported from file: %s. "
+					"Settings are about to be imported from file: %s. "
 					"Continue ?", file_name);
-				if (question_win(tmp) == YES) {
+				if (question_win(tmp, -1) == YES) {
 					if ((fp2 = g_fopen(get_datafile_full_name_from_name(CONFIG_FILE),
 						 	"wb")) != NULL) {
 						while ((c = fgetc(fp1)) != (int)EOF)
@@ -60,12 +60,12 @@ void import_params()
 						/* ????
 						modify_params2();*/
 					} else
-						warning(FALSE, 4, "Can't save Preferences to file ",
-							file_name, ": ", strerror(errno));
+						warning(BLOCK, "Can't save Settings to file '%s': %s",
+							file_name, strerror(errno));
 				}
 				fclose(fp1);
 			} else
-				warning(FALSE, 4, "Can't open ", file_name, ": ", strerror(errno));
+				warning(BLOCK, "Can't open '%s': %s", file_name, strerror(errno));
 			g_free(file_name);
 		}
 	}
@@ -78,22 +78,22 @@ void export_params()
 	GtkWidget	*dialog;
 	FILE		*fp1, *fp2;
 	char		*file_name = NULL;
-	int		exit_status = !OK;
+	int		error_code = !OK;
 	char		tmp[TMPSTR_SIZE + 1];
 	int		c;
 
 	if ((fp1 = g_fopen(get_datafile_full_name_from_name(CONFIG_FILE), "rb")) == NULL) {
 		save_to_config_file(get_params());
 		if ((fp1 = g_fopen(get_datafile_full_name_from_name(CONFIG_FILE), "rb")) == NULL) {
-			warning(FALSE, 4, "Can't open file ", get_datafile_full_name_from_name(CONFIG_FILE)
-				, ": ", strerror(errno));
+			warning(BLOCK, "Can't open file '%s': %s",
+				get_datafile_full_name_from_name(CONFIG_FILE), strerror(errno));
 			return;
 		}
 	}
 
 	gtk_window_set_keep_above(GTK_WINDOW(get_ticker_env()->win), FALSE);
 
-	dialog = gtk_file_chooser_dialog_new("Export Preferences", GTK_WINDOW(get_ticker_env()->win),
+	dialog = gtk_file_chooser_dialog_new("Export Settings", GTK_WINDOW(get_ticker_env()->win),
 			GTK_FILE_CHOOSER_ACTION_SAVE,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL_CLOSE,
 			GTK_STOCK_OK, GTK_RESPONSE_OK,
@@ -117,17 +117,17 @@ void export_params()
 				while ((c = fgetc(fp1)) != (int)EOF)
 					fputc((int)c, fp2);
 				fclose(fp2);
-				exit_status = OK;
+				error_code = OK;
 			} else
-				warning(FALSE, 4, "Can't save Preferences to file ",
-					file_name, ": ", strerror(errno));
+				warning(BLOCK, "Can't save Settings to file '%s': %s",
+					file_name, strerror(errno));
 		}
 	}
 	fclose(fp1);
 	gtk_widget_destroy(dialog);
-	if (exit_status == OK) {
+	if (error_code == OK) {
 		snprintf(tmp, TMPSTR_SIZE + 1,
-			"\nPreferences have been exported to file: %s\n", file_name);
+			"\nSettings have been exported to file: %s\n", file_name);
 		info_win("", tmp, INFO, FALSE);
 	}
 	if (file_name != NULL)
@@ -135,144 +135,51 @@ void export_params()
 	check_main_win_always_on_top();
 }
 
-#ifndef G_OS_WIN32
-void online_help()
+/* Doesn't use get_params()->open_link_args (which is done in open_link()) */
+void open_url_in_browser(char *url)
 {
-	char		tmp1[FILE_NAME_MAXLEN + 1];
-	char		*help_url = SUPPORT_URL;
-	char		*argv[3];
-	GPid		pid;
-	GError		*error = NULL;
+	char	tmp1[FILE_NAME_MAXLEN + 1];
+	char	*argv[3];
+	GPid	pid;
+	GError	*error = NULL;
 
 	if (get_params()->open_link_cmd[0] == '\0') {
-		/* TODO: should look for default browser */
-		warning(FALSE, 2, "Can't launch Browser: no command is defined.\n",
-			"Please set the 'Open in Browser' option in the Preferences window.");
-		return;
+		easily_link_with_browser();
+		if (get_params()->open_link_cmd[0] == '\0') {
+			warning(BLOCK, "Can't launch Browser: no command is defined.\n",
+				"Please set the 'Open in Browser' option in the Full Settings window.");
+			return;
+		}
 	}
-	fprintf(STD_OUT, "Spawning: %s %s\n", get_params()->open_link_cmd, help_url);
+	/* Launch browser */
+	INFO_OUT("Spawning: %s %s\n", get_params()->open_link_cmd, url)
 	str_n_cpy(tmp1, get_params()->open_link_cmd, FILE_NAME_MAXLEN);
 	argv[0] = tmp1;
-	argv[1] = help_url;
+	argv[1] = url;
 	argv[2] = NULL;
 	if (!g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &pid, &error)) {
-		warning(FALSE, 4, APP_NAME ": Can't create process ", argv[0], " - ", error->message);
-		info_win("", "Please check the 'Open in Browser' option in the Preferences window.",
+		warning(BLOCK, "%s: Can't create process %s - %s", APP_NAME, argv[0], error->message);
+		info_win("", "Please check the 'Open in Browser' option in the Full Settings window.",
 			INFO_WARNING, FALSE);
 	} else
 		g_spawn_close_pid(pid);
 }
-#else
+
 void online_help()
 {
-	char		*browser_cmd_p, browser_cmd[512], tmp[1024];
-	unsigned int	i, count, start = 0, end = 0;
-	char		link[] = SUPPORT_URL;
-	char		*argv[3];
-	GPid		pid;
-	GError		*error = NULL;
-
-	if ((browser_cmd_p = (char *)get_default_browser_from_win32registry()) != NULL) {
-		/* Find 1st str in browser_cmd */
-		str_n_cpy(tmp, browser_cmd_p, 511);
-		for (i = 0, count = 0; i < strlen(tmp); i++) {
-			if (tmp[i] == '"') {
-				count++;
-				if (count == 1)
-					start = i + 1;
-				else if (count == 2)
-					end = i - 1;
-			}
-		}
-		str_n_cpy(browser_cmd, tmp + start, MIN((end - start + 1), 511));
-		/* Launch browser */
-		fprintf(STD_OUT, "Spawning: %s %s\n", browser_cmd, link);
-		argv[0] = browser_cmd;
-		argv[1] = link;
-		argv[2] = NULL;
-		if (!g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-				NULL, NULL, &pid, &error))
-			warning(FALSE, 4, APP_NAME ": Can't create process ", argv[0],
-				" - ", error->message);
-		else
-			g_spawn_close_pid(pid);
-	}
+	open_url_in_browser(SUPPORT_URL);
 }
-#endif
 
-/*
- * This func's prototype is in libetm / error.h.
- * This func is not defined in libetm but in app and it handles errors
- * from libetm and app.
- *
- * Max number of strings = N_STR_MAX, max string length = STR_MAXLEN.
- * Outstanding stuff (strings and chars) will be ignored.
- */
-#define N_STR_MAX	8
-#define STR_MAXLEN	FILE_NAME_MAXLEN
-
-int big_error(int big_error_code, int n_str, ...)
+void go_to_paypal()
 {
-	char		error_str[64 + 32 + (N_STR_MAX * STR_MAXLEN) + 32 + 1];
-	va_list	str;
-	int		i;
-
-	get_ticker_env()->suspend_rq = TRUE;
-
-	error_str[0] = '\0';
-	str_n_cpy(error_str, "\nOuch !!  Something went wrong...\n\nCritical error in ", 64);
-	if (big_error_code > LIBETM_LASTERRORCODE)
-		str_n_cat(error_str, APP_NAME ": ", 32);
-	n_str = MIN(n_str, N_STR_MAX);
-	va_start(str, n_str);
-	for (i = 0; i < n_str ; i++)
-		str_n_cat(error_str, va_arg(str, char *), STR_MAXLEN);
-	va_end(str);
-	str_n_cat(error_str, " - Will quit now", 32);
-
-	if (STD_ERR != NULL) {
-		fprintf(STD_ERR, "%s\n", error_str);
-		fflush(NULL);
-	}
-	/* We want this win to always popup */
-	get_params()->disable_popups = 'n';
-	info_win(APP_NAME " - Critical error", error_str, INFO_ERROR, FALSE);
-	if (get_resource()->fp != NULL)
-		fclose(get_resource()->fp);
-	free2(get_ticker_env());
-	free2(get_resource());
-	free2(get_params());
-	exit(big_error_code);
+	open_url_in_browser(DONATE_WITH_PAYPAL_URL);
 }
 
-/* Will popup and wait for INFO_WIN_WAIT_TIMEOUT ms if wait == TRUE, then close
- * otherwise, will block until an appropriate keyboard/mouse action happens. */
-void warning(gboolean wait, int n_str, ...)
-{
-	char		warning_str[N_STR_MAX * STR_MAXLEN + 1];
-	va_list	str;
-	int		i;
-
-	warning_str[0] = '\0';
-	n_str = MIN(n_str, N_STR_MAX);
-	va_start(str, n_str);
-	for (i = 0; i < n_str ; i++)
-		str_n_cat(warning_str, va_arg(str, char *), STR_MAXLEN);
-	va_end(str);
-
-	fprintf(STD_ERR, "%s\n", warning_str);
-	fflush(STD_ERR);
-	if (!wait)
-		info_win("", warning_str, INFO_WARNING, FALSE);
-	else
-		info_win_wait(warning_str, INFO_WIN_WAIT_TIMEOUT);
-}
+#define NFAMMAX		(2 * 1024)
+#define NFACEMAX	64
 
 void dump_font_list()
 {
-#define NFAMMAX		1024
-#define NFACEMAX	16
-
 	PangoFontFamily		**font_fam;
 	PangoFontFamily		*font_fam2[NFAMMAX];
 	PangoFontFace		**font_face;
@@ -325,3 +232,20 @@ void dump_font_list()
 	else
 		fprintf(STD_OUT, "(%d available fonts)\n", k);
 }
+
+void dump_config()
+{
+	char	*tmp;
+	GError	*error = NULL;
+
+	fprintf(STD_OUT, APP_NAME " config (from file):\n");
+	if (g_file_get_contents(get_datafile_full_name_from_name(CONFIG_FILE), &tmp, NULL, &error)) {
+		fprintf(STD_OUT, "%s", tmp);
+		g_free(tmp);
+	} else if (error != NULL) {
+		fprintf(STD_ERR, "%s\n", error->message);
+		g_error_free(error);
+	} else
+		fprintf(STD_ERR, "Can't open file '%s': %s\n",
+			get_datafile_full_name_from_name(CONFIG_FILE), strerror(errno));
+}
