@@ -1,5 +1,5 @@
 /*
- *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2013
+ *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2020
  *	<manutm007@gmail.com>
  *
  * 	This program is free software: you can redistribute it and/or modify
@@ -20,14 +20,15 @@
 
 void set_default_options(Params *prm)
 {
-	char	tmp_font[FONT_MAXLEN + 1];
+	char tmp_font[FONT_MAXLEN + 1];
 
 	prm->delay = DELAY;
 	prm->shift_size = SHIFTSIZE;
 
 	compact_font(tmp_font, FONTNAME, FONTSIZE);
-	str_n_cpy(prm->font_n_s, tmp_font, FONT_MAXLEN);
+	str_n_cpy(prm->font_name_size, tmp_font, FONT_MAXLEN);
 	get_gdk_color_from_hexstr(FGCOLOR, &prm->fg_color, &prm->fg_color_alpha);
+	/*get_gdk_color_from_hexstr(HIGHLIGHTFGCOLOR, &prm->highlight_fg_color, &prm->highlight_fg_color_alpha);*/
 	get_gdk_color_from_hexstr(BGCOLOR, &prm->bg_color, &prm->bg_color_alpha);
 	prm->set_gradient_bg = SETGRADIENTBG;
 	get_gdk_color_from_hexstr(BGCOLOR2, &prm->bg_color2, &prm->bg_color2_alpha);
@@ -41,23 +42,23 @@ void set_default_options(Params *prm)
 	prm->windec = WINDEC;
 	prm->always_on_top = ALWAYSONTOP;
 	prm->win_transparency = WINTRANSPARENCY;
+	prm->icon_in_taskbar = ICONINTASKBAR;
+	prm->win_sticky = WINSTICKY;
+	prm->override_redirect = OVERRIDE_REDIRECT;
 
 	prm->shadow = SHADOW;
 	prm->shadow_offset_x = SHADOWOFFSET_X;
 	prm->shadow_offset_y = SHADOWOFFSET_Y;
 	prm->shadow_fx = SHADOWFX;
 
-	prm->icon_in_taskbar = ICONINTASKBAR;
-	prm->win_sticky = WINSTICKY;
-
 	str_n_cpy(prm->line_delimiter, LINEDELIMITER, DELIMITER_MAXLEN);
-	str_n_cpy(prm->cutline_delimiter, CUTLINEDELIMITER, DELIMITER_MAXLEN);
 
 	prm->special_chars = SPECIALCHARS;
 	prm->new_page_char = NEWPG;
 	prm->tab_char = TABCH;
 
 	prm->rss_refresh = RSSREFRESH;
+	prm->reverse_sc = REVERSESCROLLING;
 	prm->feed_title = FEEDTITLE;
 	str_n_cpy(prm->feed_title_delimiter, FEEDTITLEDELIMITER, DELIMITER_MAXLEN);
 	prm->item_title = ITEMTITLE;
@@ -66,6 +67,7 @@ void set_default_options(Params *prm)
 	str_n_cpy(prm->item_description_delimiter, ITEMDESCRIPTIONDELIMITER, DELIMITER_MAXLEN);
 
 	prm->n_items_per_feed = NITEMSPERFEED;
+	/*prm->mark_item_action = MARKITEMACTION;*/
 	prm->strip_html_tags = STRIPHTMLTAGS;
 	prm->upper_case_text = UPPERCASETEXT;
 	str_n_cpy(prm->homefeed, HOMEFEED, FILE_NAME_MAXLEN);
@@ -74,18 +76,21 @@ void set_default_options(Params *prm)
 	str_n_cpy(prm->open_link_args, OPENLINKARGS, FILE_NAME_MAXLEN);
 
 	prm->clock = CLOCK;
+	prm->clock_sec = CLOCKSEC;
+	prm->clock_12h = CLOCK12H;
+	prm->clock_date = CLOCKDATE;
 	get_gdk_color_from_hexstr(CFGCOLOR, &prm->clock_fg_color, &prm->clock_fg_color_alpha);
 	get_gdk_color_from_hexstr(CBGCOLOR, &prm->clock_bg_color, &prm->clock_bg_color_alpha);
 	prm->set_clock_gradient_bg = SETCLOCKGRADIENTBG;
 	get_gdk_color_from_hexstr(CBGCOLOR2, &prm->clock_bg_color2, &prm->clock_bg_color2_alpha);
 	compact_font(tmp_font, CFONTNAME, CFONTSIZE);
-	str_n_cpy(prm->clock_font_n_s, tmp_font, FONT_MAXLEN);
+	str_n_cpy(prm->clock_font_name_size, tmp_font, FONT_MAXLEN);
 
 	prm->disable_popups = DISABLEPOPUPS;
 	prm->pause_on_mouseover = PAUSEONMOUSEOVER;
 	prm->disable_leftclick = DISABLELEFTCLICK;
-	prm->mouse_wheel_scroll = MOUSEWHEELSCROLL;
-	prm->sfpicker_closes_when_pointer_leaves = SFPICKERCLOSESWHENPOINTERLEAVES;
+	prm->mouse_wheel_action = MOUSEWHEELACTION;
+	prm->sfeedpicker_autoclose = SFEEDPICKERAUTOCLOSE;
 	prm->enable_feed_ordering = ENABLEFEEDORDERING;
 
 	prm->use_authentication = USEAUTHENTICATION;
@@ -95,6 +100,8 @@ void set_default_options(Params *prm)
 	str_n_cpy(prm->proxy_port, PROXYPORT, PROXY_PORT_MAXLEN);
 	prm->use_proxy_authentication = USEPROXYAUTHENTICATION;
 	str_n_cpy(prm->proxy_user, PROXYUSER, PROXY_USER_MAXLEN);
+	prm->connect_timeout = CONNECT_TIMEOUT;
+	prm->send_recv_timeout = SENDRECV_TIMEOUT;
 }
 
 /*
@@ -116,16 +123,16 @@ int get_config_file_options(Params *prm)
 		for (i = 0; ; i++) {
 			/*
 			 * Here fgets() read OPTION_MAXLEN + 1 char max including
-			 * '\n' then add '\0' (if I'm right...)
+			 * '\n' then add '\0' (if I'm right)
 			 */
 			if (i >= N_OPTION_MAX) {
 				ptr[i] = NULL;
-				warning(FALSE, 5, "Too many lines in configuration file: ",
-					get_datafile_full_name_from_name(CONFIG_FILE),
-					"(max = ", itoa2(N_OPTION_MAX), ")"
-					"\nWon't parse further than limit\n");
+				warning(BLOCK, "Too many lines in configuration file '%s' "
+					"(max = %d)\nWon't parse further than limit\n",
+					get_datafile_full_name_from_name(CONFIG_FILE), N_OPTION_MAX);
 				break;
 			}
+			/* TODO: Should use getline() instead ? */
 			if (fgets(options_array[i], OPTION_MAXLEN + 2, conf_fp) == NULL) {
 				ptr[i] = NULL;
 				break;
@@ -141,8 +148,8 @@ int get_config_file_options(Params *prm)
 			j = 0;
 		fclose(conf_fp);
 	} else
-		fprintf(STD_ERR, "Can't read configuration file: %s - %s\n",
-			get_datafile_full_name_from_name(CONFIG_FILE), strerror(errno));
+		INFO_ERR("Can't read configuration file '%s': %s\n",
+			get_datafile_full_name_from_name(CONFIG_FILE), strerror(errno))
 	return j;
 }
 
@@ -152,41 +159,45 @@ int get_config_file_options(Params *prm)
  */
 int parse_options_array(Params *prm, int n_options, const char *option[])
 {
-	int	i, exit_status = OK;
+	int	i, j, error_code = OK;
 
 	if (n_options > N_OPTION_MAX) {
 		n_options = N_OPTION_MAX;
-		fprintf(STD_ERR,
-			"Too many options (max = %d) - Won't parse further than limit\n",
-			N_OPTION_MAX);
-			exit_status = OPTION_TOO_MANY;
+		INFO_ERR("Too many options (max = %d) - Won't parse further than limit\n",
+			N_OPTION_MAX)
+		error_code = OPTION_TOO_MANY;
 	}
 	for (i = 0; i < n_options && option[i] != NULL; i++)
-		if (get_name_value_from_option(prm, option[i]) != OK)
-			exit_status = OPTION_ERROR;
-	return exit_status;
+		if ((j = get_name_value_from_option(prm, option[i])) != OK) {
+			/* FIXME: Will only get last error code */
+			error_code = j;
+			/* But if OPTION_ERROR_QUIT is set to TRUE */
+			if (OPTION_ERROR_QUIT)
+				break;
+		}
+	return error_code;
 }
 
 int get_name_value_from_option(Params *prm, const char *option)
 {
 	/*
 	 * Options are splitted into name and value.
-	 * font -> fname is FONT_NAME_MAXLEN char max long, fsize FONT_SIZE_MAXLEN,
-	 * plus 1 extra space so font_n_s (font name size) is up to
+	 * font -> font_name is FONT_NAME_MAXLEN char max long, font_size FONT_SIZE_MAXLEN,
+	 * plus 1 extra space so font_name_size is up to
 	 * FONT_MAXLEN = FONT_NAME_MAXLEN + 1 + FONT_SIZE_MAXLEN char long.
 	 * OPTION_MAXLEN = OPTION_NAME_MAXLEN + OPTION_VALUE_MAXLEN + 4 (-name="value").
 	 *
 	 * === If unsure, check this in tickr.h ===
 	 */
 	char	name[OPTION_NAME_MAXLEN + 1], value[OPTION_VALUE_MAXLEN + 1];
-	char	fname[FONT_NAME_MAXLEN + 1], fsize[FONT_SIZE_MAXLEN + 1];
-	char	cfname[FONT_NAME_MAXLEN + 1], cfsize[FONT_SIZE_MAXLEN + 1];
+	char	font_name[FONT_NAME_MAXLEN + 1], font_size[FONT_SIZE_MAXLEN + 1];
+	char	c_font_name[FONT_NAME_MAXLEN + 1], c_font_size[FONT_SIZE_MAXLEN + 1];
 	char	tmp[OPTION_MAXLEN + 1];
 	char	tmp_font[FONT_MAXLEN + 1];
-	int	i, exit_status = OK;
+	int	i, error_code = OK;
 
-	split_font(prm->font_n_s, fname, fsize);
-	split_font(prm->clock_font_n_s, cfname, cfsize);
+	split_font(prm->font_name_size, font_name, font_size);
+	split_font(prm->clock_font_name_size, c_font_name, c_font_size);
 	/*
 	 * Split option into name & value if any
 	 */
@@ -202,8 +213,8 @@ int get_name_value_from_option(Params *prm, const char *option)
 				break;
 			}
 	} else if (option[0] != '\0') {
-		fprintf(STD_ERR, "Invalid option: %s\n", option);
-		return (exit_status = OPTION_INVALID);
+		INFO_ERR("Invalid option: %s\n", option)
+		return (error_code = OPTION_INVALID);
 	}
 	/*
 	 * All *string* values are saved inclosed in a pair of ". When read, enclosing " (or ')
@@ -223,32 +234,39 @@ int get_name_value_from_option(Params *prm, const char *option)
 	else if (strcmp(name, "shiftsize") == 0)
 		prm->shift_size = atoi(value);
 	else if (strcmp(name, "fontname") == 0)
-		str_n_cpy(fname, value, FONT_NAME_MAXLEN);
+		str_n_cpy(font_name, value, FONT_NAME_MAXLEN);
 	else if (strcmp(name, "fontsize") == 0) {
-		str_n_cpy(fsize, value, FONT_SIZE_MAXLEN);
-		if (atoi(fsize) > FONT_MAXSIZE) {
-			snprintf(fsize, FONT_SIZE_MAXLEN + 1, "%3d", FONT_MAXSIZE);
-			fprintf(STD_ERR, "Font size set to %d (can't be above)\n",
-				FONT_MAXSIZE);
+		str_n_cpy(font_size, value, FONT_SIZE_MAXLEN);
+		if (atoi(font_size) > FONT_MAXSIZE) {
+			snprintf(font_size, FONT_SIZE_MAXLEN + 1, "%3d", FONT_MAXSIZE);
+			INFO_ERR("Font size set to %d (can't be above)\n",
+				FONT_MAXSIZE)
 		}
 	} else if (strcmp(name, "fgcolor") == 0) {
 		if (!get_gdk_color_from_hexstr(value, &prm->fg_color,
 						&prm->fg_color_alpha)) {
 			get_gdk_color_from_hexstr(FGCOLOR, &prm->fg_color,
 						&prm->fg_color_alpha);
-			exit_status = OPTION_VALUE_INVALID;
+			error_code = OPTION_VALUE_INVALID;
 		}
-	} else if (strcmp(name, "bgcolor") == 0) {
+	} else /*if (strcmp(name, "highlightfgcolor") == 0) {
+		if (!get_gdk_color_from_hexstr(value, &prm->highlight_fg_color,
+						&prm->highlight_fg_color_alpha)) {
+			get_gdk_color_from_hexstr(HIGHLIGHTFGCOLOR, &prm->highlight_fg_color,
+						&prm->highlight_fg_color_alpha);
+			error_code = OPTION_VALUE_INVALID;
+		}
+	} else*/ if (strcmp(name, "bgcolor") == 0) {
 		if (!get_gdk_color_from_hexstr(value, &prm->bg_color,
 						&prm->bg_color_alpha)) {
 			get_gdk_color_from_hexstr(BGCOLOR, &prm->bg_color,
 						&prm->bg_color_alpha);
-			exit_status = OPTION_VALUE_INVALID;
+			error_code = OPTION_VALUE_INVALID;
 		}
 	} else if (strcmp(name, "setgradientbg") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->set_gradient_bg = value[0];
 	} else if (strcmp(name, "bgcolor2") == 0) {
@@ -256,12 +274,12 @@ int get_name_value_from_option(Params *prm, const char *option)
 						&prm->bg_color2_alpha)) {
 			get_gdk_color_from_hexstr(BGCOLOR, &prm->bg_color2,
 						&prm->bg_color2_alpha);
-			exit_status = OPTION_VALUE_INVALID;
+			error_code = OPTION_VALUE_INVALID;
 		}
 	} else if (strcmp(name, "disablescreenlimits") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->disable_screen_limits = value[0];
 	} else if (strcmp(name, "win_x") == 0)
@@ -283,65 +301,67 @@ int get_name_value_from_option(Params *prm, const char *option)
 			prm->win_h = DRWA_HEIGHT_MIN;
 	} else if (strcmp(name, "windec") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->windec = value[0];
 	} else if (strcmp(name, "alwaysontop") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->always_on_top = value[0];
 	} else if (strcmp(name, "wintransparency") == 0) {
-		/* Compare as integers (because less bug-prone) then set as gdouble.
+		/* Compare as integers (because less bug-prone) then set as double.
 		 * Saved to / retrieved from config file as [transparency x 10]. */
 		prm->win_transparency = atoi(value);
 		if ((int)prm->win_transparency < 1) {
-			fprintf(STD_ERR, "Invalid value: %s - will be set to 1\n", option);
+			INFO_ERR("Invalid value: %s - will be set to 1\n", option)
 			prm->win_transparency = 1.0;
-			exit_status = OPTION_VALUE_INVALID;
+			error_code = OPTION_VALUE_INVALID;
 		} else if ((int)prm->win_transparency > 10) {
-			fprintf(STD_ERR, "Invalid value: %s - will be set to 10\n", option);
+			INFO_ERR("Invalid value: %s - will be set to 10\n", option)
 			prm->win_transparency = 10.0;
-			exit_status = OPTION_VALUE_INVALID;
+			error_code = OPTION_VALUE_INVALID;
 		}
 		prm->win_transparency /= 10;
 	} else if (strcmp(name, "iconintaskbar") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->icon_in_taskbar = value[0];
 	} else if (strcmp(name, "winsticky") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->win_sticky = value[0];
+	} else if (strcmp(name, "overrideredirect") == 0) {
+		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
+		} else
+			prm->override_redirect = value[0];
 	} else if (strcmp(name, "shadow") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->shadow = value[0];
 	}
-	/* Should check value here */
-	else if (strcmp(name, "shadowoffset_x") == 0)
+	else if (strcmp(name, "shadowoffset_x") == 0)	/* Should check value here */
 		prm->shadow_offset_x = atoi(value);
-	/* Should check value here */
-	else if (strcmp(name, "shadowoffset_y") == 0)
+	else if (strcmp(name, "shadowoffset_y") == 0)	/* Should check value here */
 		prm->shadow_offset_y = atoi(value);
 	else if (strcmp(name, "shadowfx") == 0)
 		prm->shadow_fx = atoi(value);
 	else if (strcmp(name, "linedelimiter") == 0)
 		str_n_cpy(prm->line_delimiter, value, DELIMITER_MAXLEN);
-	else if (strcmp(name, "cutlinedelimiter") == 0)
-		str_n_cpy(prm->cutline_delimiter, value, DELIMITER_MAXLEN);
 	else if (strcmp(name, "specialchars") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->special_chars = value[0];
 	} else if (strcmp(name, "newpgchar") == 0)
@@ -350,42 +370,55 @@ int get_name_value_from_option(Params *prm, const char *option)
 		prm->tab_char = value[0];
 	else if (strcmp(name, "rssrefresh") == 0)
 		prm->rss_refresh = atoi(value);
-	else if (strcmp(name, "feedtitle") == 0) {
+	else if (strcmp(name, "revsc") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
+		} else
+			prm->reverse_sc = value[0];
+	} else if (strcmp(name, "feedtitle") == 0) {
+		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->feed_title = value[0];
 	} else if (strcmp(name, "feedtitledelimiter") == 0)
 		str_n_cpy(prm->feed_title_delimiter, value, DELIMITER_MAXLEN);
 	else if (strcmp(name, "itemtitle") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->item_title = value[0];
 	} else if (strcmp(name, "itemtitledelimiter") == 0)
 		str_n_cpy(prm->item_title_delimiter, value, DELIMITER_MAXLEN);
 	else if (strcmp(name, "itemdescription") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->item_description = value[0];
 	} else if (strcmp(name, "itemdescriptiondelimiter") == 0)
 		str_n_cpy(prm->item_description_delimiter, value, DELIMITER_MAXLEN);
 	else if (strcmp(name, "nitemsperfeed") == 0)
 		prm->n_items_per_feed = atoi(value);
-	else if (strcmp(name, "rmtags") == 0) {
+	else /*if (strcmp(name, "markitemaction") == 0) {
+		if (strcmp(value, "h") != 0 && strcmp(value, "c") != 0 \
+					&& strcmp(value, "n") != 0) {
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
+		} else
+			prm->mark_item_action = value[0];
+	} else*/ if (strcmp(name, "rmtags") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->strip_html_tags = value[0];
 	} else if (strcmp(name, "uppercasetext") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->upper_case_text = value[0];
 
@@ -398,32 +431,50 @@ int get_name_value_from_option(Params *prm, const char *option)
 	else if (strcmp(name, "clock") == 0) {
 		if (strcmp(value, "l") != 0 && strcmp(value, "r") != 0 \
 					&& strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->clock = value[0];
+	} else if (strcmp(name, "clocksec") == 0) {
+		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
+		} else
+			prm->clock_sec = value[0];
+	} else if (strcmp(name, "clock12h") == 0) {
+		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
+		} else
+			prm->clock_12h = value[0];
+	} else if (strcmp(name, "clockdate") == 0) {
+		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
+		} else
+			prm->clock_date = value[0];
 	} else if (strcmp(name, "clockfontname") == 0)
-		str_n_cpy(cfname, value, FONT_NAME_MAXLEN);
+		str_n_cpy(c_font_name, value, FONT_NAME_MAXLEN);
 	else if (strcmp(name, "clockfontsize") == 0)
-		str_n_cpy(cfsize, value, FONT_SIZE_MAXLEN);
+		str_n_cpy(c_font_size, value, FONT_SIZE_MAXLEN);
 	else if (strcmp(name, "clockfgcolor") == 0) {
 		if (!get_gdk_color_from_hexstr(value, &prm->clock_fg_color,
 					&prm->clock_fg_color_alpha)) {
 			get_gdk_color_from_hexstr(CFGCOLOR, &prm->clock_fg_color,
 					&prm->clock_fg_color_alpha);
-			exit_status = OPTION_VALUE_INVALID;
+			error_code = OPTION_VALUE_INVALID;
 		}
 	} else if (strcmp(name, "clockbgcolor") == 0) {
 		if (!get_gdk_color_from_hexstr(value, &prm->clock_bg_color,
 					&prm->clock_bg_color_alpha)) {
 			get_gdk_color_from_hexstr(CBGCOLOR, &prm->clock_bg_color,
 					&prm->clock_bg_color_alpha);
-			exit_status = OPTION_VALUE_INVALID;
+			error_code = OPTION_VALUE_INVALID;
 		}
 	} else if (strcmp(name, "setclockgradientbg") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->set_clock_gradient_bg = value[0];
 	} else if (strcmp(name, "clockbgcolor2") == 0) {
@@ -431,50 +482,50 @@ int get_name_value_from_option(Params *prm, const char *option)
 						&prm->clock_bg_color2_alpha)) {
 			get_gdk_color_from_hexstr(BGCOLOR, &prm->clock_bg_color2,
 						&prm->clock_bg_color2_alpha);
-			exit_status = OPTION_VALUE_INVALID;
+			error_code = OPTION_VALUE_INVALID;
 		}
 	} else if (strcmp(name, "disablepopups") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->disable_popups = value[0];
 	} else if (strcmp(name, "pauseonmouseover") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->pause_on_mouseover = value[0];
 	} else if (strcmp(name, "disableleftclick") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->disable_leftclick = value[0];
-	} else if (strcmp(name, "mousewheelscroll") == 0) {
+	} else if (strcmp(name, "mousewheelaction") == 0) {
 		if (strcmp(value, "s") != 0 && strcmp(value, "f") != 0 \
 					&& strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
-			prm->mouse_wheel_scroll = value[0];
-	} else if (strcmp(name, "sfpickercloseswhenpointerleaves") == 0) {
+			prm->mouse_wheel_action = value[0];
+	} else if (strcmp(name, "sfeedpickerautoclose") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
-			prm->sfpicker_closes_when_pointer_leaves = value[0];
+			prm->sfeedpicker_autoclose = value[0];
 	} else if (strcmp(name, "enablefeedordering") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else
 			prm->enable_feed_ordering = value[0];
 	/* Params in connection settings win */
 	} else if (strcmp(name, "useauth") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else {
 			prm->use_authentication = value[0];
 			set_use_authentication(value[0] == 'y' ? TRUE : FALSE);
@@ -487,8 +538,8 @@ int get_name_value_from_option(Params *prm, const char *option)
 		str_n_cpy(get_http_auth_psw(), value, PSW_MAXLEN);
 	else if (strcmp(name, "useproxy") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else {
 			prm->use_proxy = value[0];
 			set_use_proxy(value[0] == 'y' ? TRUE : FALSE);
@@ -501,8 +552,8 @@ int get_name_value_from_option(Params *prm, const char *option)
 		str_n_cpy(get_proxy_port(), value, PROXY_PORT_MAXLEN);
 	} else if (strcmp(name, "useproxyauth") == 0) {
 		if (strcmp(value, "y") != 0 && strcmp(value, "n") != 0) {
-			fprintf(STD_ERR, "Invalid value: %s\n", option);
-			exit_status = OPTION_VALUE_INVALID;
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
 		} else {
 			prm->use_proxy_authentication = value[0];
 			set_use_proxy_auth(value[0] == 'y' ? TRUE : FALSE);
@@ -513,34 +564,45 @@ int get_name_value_from_option(Params *prm, const char *option)
 	/* This option is never saved */
 	} else if (strcmp(name, "proxypsw") == 0)
 		str_n_cpy(get_proxy_auth_psw(), value, PROXY_PSW_MAXLEN);
+	else if (strcmp(name, "connect-timeout") == 0) {
+		if (atoi(value) < 1 || atoi(value) > 60) {
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
+		} else
+			prm->connect_timeout = atoi(value);
+	} else if (strcmp(name, "sendrecv-timeout") == 0) {
+		if (atoi(value) < 1 || atoi(value) > 60) {
+			INFO_ERR("Invalid value: %s\n", option)
+			error_code = OPTION_VALUE_INVALID;
+		} else
+			prm->send_recv_timeout = atoi(value);
 	/* Connect params until here */
-	/* This option is only set from command line */
-	else if (strcmp(name, "instance-id") == 0)
-		;/* Does nothing but avoid an 'unknown option' warning/error */
-	/* This option is only set from command line */
-	else if (strcmp(name, "no-ui") == 0)
-		;/* Does nothing but avoid an 'unknown option' warning/error */
-	else {
-		fprintf(STD_ERR, "Unknown option: %s\n", option);
-		exit_status = OPTION_UNKNOWN;
+	/* Following options are only set from command line (and never saved) */
+	} else if (strcmp(name, "instance-id") == 0) {
+		/* Does nothing but avoid an 'unknown option' warning/error */
+	} else if (strcmp(name, "no-ui") == 0) {
+		/* Does nothing but avoid an 'unknown option' warning/error */
+	} else {
+		INFO_ERR("Unknown option: %s\n", option)
+		error_code = OPTION_UNKNOWN;
 	}
-	compact_font(tmp_font, fname, fsize);
-	str_n_cpy(prm->font_n_s, tmp_font, FONT_MAXLEN);
-	compact_font(tmp_font, cfname, cfsize);
-	str_n_cpy(prm->clock_font_n_s, tmp_font, FONT_MAXLEN);
-	return exit_status;
+	compact_font(tmp_font, font_name, font_size);
+	str_n_cpy(prm->font_name_size, tmp_font, FONT_MAXLEN);
+	compact_font(tmp_font, c_font_name, c_font_size);
+	str_n_cpy(prm->clock_font_name_size, tmp_font, FONT_MAXLEN);
+	return error_code;
 }
 
 void save_to_config_file(const Params *prm)
 {
 	FILE		*conf_fp;
-	char		fname[FONT_NAME_MAXLEN + 1], fsize[FONT_SIZE_MAXLEN + 1];
-	char		cfname[FONT_NAME_MAXLEN + 1], cfsize[FONT_SIZE_MAXLEN + 1];
+	char		font_name[FONT_NAME_MAXLEN + 1], font_size[FONT_SIZE_MAXLEN + 1];
+	char		c_font_name[FONT_NAME_MAXLEN + 1], c_font_size[FONT_SIZE_MAXLEN + 1];
 	char		tmp[4 * 1024];
 
 	if ((conf_fp = g_fopen(get_datafile_full_name_from_name(CONFIG_FILE), "wb")) != NULL) {
-		split_font(prm->font_n_s, fname, fsize);
-		split_font(prm->clock_font_n_s, cfname, cfsize);
+		split_font(prm->font_name_size, font_name, font_size);
+		split_font(prm->clock_font_name_size, c_font_name, c_font_size);
 		snprintf(tmp, 4 * 1024, "%s%s%s",
 			/* All *string* values are saved inclosed in a pair of ".
 			 */
@@ -549,6 +611,7 @@ void save_to_config_file(const Params *prm)
 			"-fontname=\"%s\"\n"
 			"-fontsize=%d\n"
 			"-fgcolor=\"%s\"\n"
+			/*"-highlightfgcolor=\"%s\"\n"*/
 			"-bgcolor=\"%s\"\n"
 			"-setgradientbg=%c\n"
 			"-bgcolor2=\"%s\"\n"
@@ -562,16 +625,17 @@ void save_to_config_file(const Params *prm)
 			"-wintransparency=%d\n"
 			"-iconintaskbar=%c\n"
 			"-winsticky=%c\n"
+			"-overrideredirect=%c\n"
 			"-shadow=%c\n"
-			"-shadowoffset_x=%d\n",
+			"-shadowoffset_x=%d\n"
 			"-shadowoffset_y=%d\n"
 			"-shadowfx=%d\n"
 			"-linedelimiter=\"%s\"\n"
-			"-cutlinedelimiter=\"%s\"\n"
-			"-specialchars=%c\n"
+			"-specialchars=%c\n",
 			"-newpgchar=%c\n"
 			"-tabchar=%c\n"
 			"-rssrefresh=%d\n"
+			"-revsc=%c\n"
 			"-feedtitle=%c\n"
 			"-feedtitledelimiter=\"%s\"\n"
 			"-itemtitle=%c\n"
@@ -579,23 +643,27 @@ void save_to_config_file(const Params *prm)
 			"-itemdescription=%c\n"
 			"-itemdescriptiondelimiter=\"%s\"\n"
 			"-nitemsperfeed=%d\n"
+			/*"-markitemaction=%c\n"*/
 			"-rmtags=%c\n"
 			"-uppercasetext=%c\n"
 			"-homefeed=\"%s\"\n"
 			"-openlinkcmd=\"%s\"\n"
-			"-openlinkargs=\"%s\"\n",
+			"-openlinkargs=\"%s\"\n"
 			"-clock=%c\n"
+			"-clocksec=%c\n"
+			"-clock12h=%c\n"
+			"-clockdate=%c\n"
 			"-clockfontname=\"%s\"\n"
 			"-clockfontsize=%d\n"
 			"-clockfgcolor=\"%s\"\n"
 			"-clockbgcolor=\"%s\"\n"
 			"-setclockgradientbg=%c\n"
-			"-clockbgcolor2=\"%s\"\n"
+			"-clockbgcolor2=\"%s\"\n",
 			"-disablepopups=%c\n"
 			"-pauseonmouseover=%c\n"
 			"-disableleftclick=%c\n"
-			"-mousewheelscroll=%c\n"
-			"-sfpickercloseswhenpointerleaves=%c\n"
+			"-mousewheelaction=%c\n"
+			"-sfeedpickerautoclose=%c\n"
 			"-enablefeedordering=%c\n"
 			"-useauth=%c\n"
 			"-user=\"%s\"\n"
@@ -604,12 +672,15 @@ void save_to_config_file(const Params *prm)
 			"-proxyport=\"%s\"\n"
 			"-useproxyauth=%c\n"
 			"-proxyuser=\"%s\"\n"
+			"-connect-timeout=%d\n"
+			"-sendrecv-timeout=%d\n"
 			);
 		fprintf(conf_fp, tmp,
 			prm->delay,
 			prm->shift_size,
-			fname, atoi(fsize),
+			font_name, atoi(font_size),
 			get_hexstr_from_gdk_color(&prm->fg_color, &prm->fg_color_alpha),
+			/*get_hexstr_from_gdk_color(&prm->highlight_fg_color, &prm->highlight_fg_color_alpha),*/
 			get_hexstr_from_gdk_color(&prm->bg_color, &prm->bg_color_alpha),
 			prm->set_gradient_bg,
 			get_hexstr_from_gdk_color(&prm->bg_color2, &prm->bg_color2_alpha),
@@ -623,16 +694,17 @@ void save_to_config_file(const Params *prm)
 			(int)(prm->win_transparency * 10),
 			prm->icon_in_taskbar,
 			prm->win_sticky,
+			prm->override_redirect,
 			prm->shadow,
 			prm->shadow_offset_x,
 			prm->shadow_offset_y,
 			prm->shadow_fx,
 			prm->line_delimiter,
-			prm->cutline_delimiter,
 			prm->special_chars,
 			prm->new_page_char,
 			prm->tab_char,
 			prm->rss_refresh,
+			prm->reverse_sc,
 			prm->feed_title,
 			prm->feed_title_delimiter,
 			prm->item_title,
@@ -640,14 +712,18 @@ void save_to_config_file(const Params *prm)
 			prm->item_description,
 			prm->item_description_delimiter,
 			prm->n_items_per_feed,
+			/*prm->mark_item_action,*/
 			prm->strip_html_tags,
 			prm->upper_case_text,
 			prm->homefeed,
 			prm->open_link_cmd,
 			prm->open_link_args,
 			prm->clock,
-			cfname,
-			atoi(cfsize),
+			prm->clock_sec,
+			prm->clock_12h,
+			prm->clock_date,
+			c_font_name,
+			atoi(c_font_size),
 			get_hexstr_from_gdk_color(&prm->clock_fg_color, &prm->clock_fg_color_alpha),
 			get_hexstr_from_gdk_color(&prm->clock_bg_color, &prm->clock_bg_color_alpha),
 			prm->set_clock_gradient_bg,
@@ -655,8 +731,8 @@ void save_to_config_file(const Params *prm)
 			prm->disable_popups,
 			prm->pause_on_mouseover,
 			prm->disable_leftclick,
-			prm->mouse_wheel_scroll,
-			prm->sfpicker_closes_when_pointer_leaves,
+			prm->mouse_wheel_action,
+			prm->sfeedpicker_autoclose,
 			prm->enable_feed_ordering,
 			prm->use_authentication,
 			prm->user,
@@ -664,13 +740,14 @@ void save_to_config_file(const Params *prm)
 			prm->proxy_host,
 			prm->proxy_port,
 			prm->use_proxy_authentication,
-			prm->proxy_user
+			prm->proxy_user,
+			prm->connect_timeout,
+			prm->send_recv_timeout
 			);
 		fclose(conf_fp);
 	} else
-		warning(FALSE, 4, "Can't save configuration file ",
-			get_datafile_full_name_from_name(CONFIG_FILE), ": ",
-			strerror(errno));
+		warning(BLOCK, "Can't save configuration file '%s': %s",
+			get_datafile_full_name_from_name(CONFIG_FILE), strerror(errno));
 }
 
 /*
@@ -705,7 +782,7 @@ int get_gdk_color_from_hexstr(const char *str, GdkColor *color, guint16 *color_a
 			errflag = 1;
 	}
 	if (errflag != 0) {
-		fprintf(STD_ERR, "Invalid color: %s\n", str);
+		INFO_ERR("Invalid color: %s\n", str)
 		return FALSE;
 	} else
 		return TRUE;
@@ -723,20 +800,50 @@ const char *get_hexstr_from_gdk_color(const GdkColor *color, const guint16 *colo
 	count++;
 	count &= 63;
 	snprintf(str[count], 10, "#%02x%02x%02x%02x", color->red / 256, color->green / 256,
-					color->blue / 256, *color_alpha / 256);
+		color->blue / 256, *color_alpha / 256);
 	if (strlen(str[count]) != 9)
 		return NULL;
 	else
 		return (const  char *)str[count];
 }
 
+/*
+ * Adjust size in font_name_and_size str by computing font size from required height
+ * (str must be able to store FONT_MAXLEN chars.) Does nothing if height <= 0.
+ */
+void adjust_font_size_to_rq_height(char *font_name_and_size, int height)
+{
+	char	f_name[FONT_NAME_MAXLEN + 1];
+	char	f_size[FONT_SIZE_MAXLEN + 1];
+
+	if (height > 0) {
+		split_font(font_name_and_size, f_name, f_size);
+		snprintf(f_size, FONT_SIZE_MAXLEN + 1, "%3d", get_font_size_from_layout_height(height, f_name));
+		/* Font size can't be > FONT_MAXSIZE */
+		if (atoi(f_size) > FONT_MAXSIZE)
+			snprintf(f_size, FONT_SIZE_MAXLEN + 1, "%3d", FONT_MAXSIZE);
+		compact_font(font_name_and_size, f_name, f_size);
+	} else if (height == 0)
+		DEBUG_INFO("Required height = 0\n")
+	else
+		INFO_ERR("%s() error: Required height < 0\n", __func__)
+}
+
+int get_font_size_from_font_name_size(const char *font_name_size)
+{
+	char font_name[FONT_NAME_MAXLEN + 1], font_size[FONT_SIZE_MAXLEN + 1];
+
+	split_font(font_name_size, font_name, font_size);
+	return atoi(font_size);
+}
+
 /*
  * Space needed for font_name: FONT_NAME_MAXLEN + 1 bytes / font_size: FONT_SIZE_MAXLEN + 1 bytes
  */
 void split_font(const char *font_name_size, char *font_name, char *font_size)
 {
-	char		tmp[FONT_MAXLEN + 1];
-	int		i;
+	char	tmp[FONT_MAXLEN + 1];
+	int	i;
 
 	if (font_name_size[0] == '\0') {
 		font_name[0] = '\0';
