@@ -1,5 +1,5 @@
 /*
- *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2013
+ *	TICKR - GTK-based Feed Reader - Copyright (C) Emmanuel Thomas-Maurin 2009-2020
  *	<manutm007@gmail.com>
  *
  * 	This program is free software: you can redistribute it and/or modify
@@ -23,8 +23,8 @@ static TickerEnv	*env;
 static Params		*prm, *prm_bak;
 static GtkWidget	*qsetup;
 static GtkWidget	*top_but, *bottom_but, *spinbut_winy, *checkbut_alwaysontop;
-static GtkWidget	*checkbut_feedtitle, *checkbut_itemtitle, *checkbut_itemdes;
-static GtkWidget	*entry_openlinkcmd, *checkbut_nopopups;
+/*static GtkWidget	*checkbut_feedtitle, *checkbut_itemtitle, *checkbut_itemdes;*/
+static GtkWidget	*entry_openlinkcmd/*, *checkbut_nopopups*/;
 static	GtkObject*	adj_winy;
 
 static void qsetup_prepare(GtkWidget *widget, GtkWidget *page, gpointer data)
@@ -42,9 +42,9 @@ static void qsetup_cancel(GtkWidget *widget, gpointer data)
 	gtk_main_quit();
 }
 
-static gint qsetup_del_event()
+static int qsetup_del_event()
 {
-	if (question_win("Cancel quick setup ?") == YES) {
+	if (question_win("Cancel quick setup ?", -1) == YES) {
 		memcpy((void *)prm, (const void *)prm_bak, sizeof(Params));
 		gtk_main_quit();
 		return FALSE;
@@ -58,48 +58,48 @@ static void keep_setting(GtkWidget *widget, gpointer data)
 	switch((int)data) {
 	case 1:
 		/* Window y */
-		prm->win_y = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_winy));
+		prm->win_y = (int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinbut_winy));
 		break;
 	case 2:
 		/* Window always-on-top */
-		if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_alwaysontop)))
+		if ((zboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_alwaysontop)))
 			prm->always_on_top = 'y';
 		else
 			prm->always_on_top = 'n';
 		break;
-	case 3:
-		/* Feed title */
-		if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_feedtitle)))
+	/*case 3:
+		// Feed title
+		if ((zboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_feedtitle)))
 			prm->feed_title = 'y';
 		else
 			prm->feed_title = 'n';
 		break;
 	case 4:
-		/* Item title */
-		if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_itemtitle)))
+		// Item title
+		if ((zboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_itemtitle)))
 			prm->item_title = 'y';
 		else
 			prm->item_title = 'n';
 		break;
 	case 5:
-		/* Item decription */
-		if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_itemdes)))
+		// Item decription
+		if ((zboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_itemdes)))
 			prm->item_description = 'y';
 		else
 			prm->item_description = 'n';
-		break;
-	case 6:
+		break;*/
+	case 3/*6*/:
 		/* Open link cmd */
 		str_n_cpy(prm->open_link_cmd,
 			(char *)gtk_entry_get_text(GTK_ENTRY(entry_openlinkcmd)), FILE_NAME_MAXLEN);
 		break;
-	case 7:
-		/* Disable popups */
-		if ((gboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_nopopups)))
+	/*case 7:
+		// Disable popups
+		if ((zboolean)gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbut_nopopups)))
 			prm->disable_popups = 'y';
 		else
 			prm->disable_popups = 'n';
-		break;
+		break;*/
 	default:
 		break;
 	}
@@ -109,13 +109,13 @@ static void qsetup_apply(GtkWidget *widget, gpointer data)
 {
 	widget = widget;
 	data = data;
-	if (gtk_assistant_get_current_page(GTK_ASSISTANT(qsetup)) == 8) {
+	if (gtk_assistant_get_current_page(GTK_ASSISTANT(qsetup)) == 4/*8*/) {
 		save_to_config_file(prm);
 		gtk_main_quit();
 	}
 }
 
-static gint move_to_top(GtkWidget *widget)
+static int move_to_top(GtkWidget *widget)
 {
 	widget = widget;
 	get_params()->win_y = 0;
@@ -123,11 +123,11 @@ static gint move_to_top(GtkWidget *widget)
 	return TRUE;
 }
 
-static gint move_to_bottom(GtkWidget *widget)
+static int move_to_bottom(GtkWidget *widget)
 {
-	char	font_n_s[FONT_MAXLEN + 1];
-	char	fname[FONT_NAME_MAXLEN + 1], fsize[FONT_SIZE_MAXLEN + 1];
-	gint	requested_font_size, requested_h, y_bottom;
+	char	font_name_size[FONT_MAXLEN + 1];
+	char	font_name[FONT_NAME_MAXLEN + 1], font_size[FONT_SIZE_MAXLEN + 1];
+	int	requested_font_size, requested_h, y_bottom;
 	/* We backup this param first because... */
 	char	disable_popups_bak = get_params()->disable_popups;
 
@@ -141,27 +141,27 @@ static gint move_to_bottom(GtkWidget *widget)
 		requested_h = DRWA_HEIGHT_MIN;
 	else if (requested_h == 0) {
 		/* Compute ticker height from requested font size */
-		str_n_cpy(font_n_s, get_params()->font_n_s, FONT_MAXLEN);
-		split_font(font_n_s, fname, fsize);
+		str_n_cpy(font_name_size, get_params()->font_name_size, FONT_MAXLEN);
+		split_font(font_name_size, font_name, font_size);
 		/* In all cases, font size can't be > FONT_MAXSIZE */
-		requested_font_size =  MIN(atoi(fsize), FONT_MAXSIZE);
-		snprintf(fsize, FONT_SIZE_MAXLEN + 1, "%3d", requested_font_size);
-		compact_font(font_n_s, fname, fsize);
-		requested_h = get_layout_height_from_fnamesize(font_n_s);
+		requested_font_size =  MIN(atoi(font_size), FONT_MAXSIZE);
+		snprintf(font_size, FONT_SIZE_MAXLEN + 1, "%3d", requested_font_size);
+		compact_font(font_name_size, font_name, font_size);
+		requested_h = get_layout_height_from_font_name_size(font_name_size);
 	}
 
 	y_bottom = env->screen_h - requested_h;
 #ifndef G_OS_WIN32
 	/* How to get taskbar height on Linux ? */
-	warning(FALSE, 3, "Taskbar height: Will use ", itoa2(ARBITRARY_TASKBAR_HEIGHT),
-		" pixels arbitrary value - Please adjust as necessary afterwards");
+	warning(BLOCK, "Taskbar height: Will use %d pixels arbitrary value - "
+		"Please adjust as necessary afterwards", ARBITRARY_TASKBAR_HEIGHT);
 	y_bottom -= ARBITRARY_TASKBAR_HEIGHT;
 #else
 	if (get_win32_taskbar_height() != -1)
 		y_bottom -= get_win32_taskbar_height();
 	else {
-		warning(FALSE, 3, "Couldn't compute Taskbar height: Will use ", itoa2(ARBITRARY_TASKBAR_HEIGHT),
-			" pixels arbitrary value - Please adjust as necessary afterwards");
+		warning(BLOCK, "Couldn't compute Taskbar height: Will use %d pixels arbitrary value - "
+			"Please adjust as necessary afterwards", ARBITRARY_TASKBAR_HEIGHT);
 		y_bottom -= ARBITRARY_TASKBAR_HEIGHT;
 	}
 #endif
@@ -171,6 +171,10 @@ static gint move_to_bottom(GtkWidget *widget)
 	return TRUE;
 }
 
+/*
+ * TODO: which other ones here ????
+ * Reverse scrolling / Read n items max per feed / Mouse wheel scrolling behaviour
+ */
 void quick_setup(Params *prm0)
 {
 #define N_PAGES_MAX	16
@@ -213,7 +217,7 @@ void quick_setup(Params *prm0)
 	hbox[i] = gtk_hbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox[i]), 15);
 
-	label[i] = gtk_label_new("Y position <small>(Pixels)</small>:");
+	label[i] = gtk_label_new("Vertical Location <small>(Pixels)</small>:");
 	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
 	gtk_box_pack_start(GTK_BOX(hbox[i]), label[i], FALSE, FALSE, 0);
@@ -242,7 +246,7 @@ void quick_setup(Params *prm0)
 	hbox[i] = gtk_hbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox[i]), 15);
 
-	label[i] = gtk_label_new("Window Always-On-Top:");
+	label[i] = gtk_label_new(APP_NAME " always above other windows:");
 	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
 	gtk_box_pack_start(GTK_BOX(hbox[i]), label[i], FALSE, FALSE, 0);
@@ -266,7 +270,8 @@ void quick_setup(Params *prm0)
 	gtk_assistant_set_page_title(GTK_ASSISTANT(qsetup), page[i], APP_NAME " always-on-top");
 	i++;
 
-	/* Feed title */
+/* Following 3 pages now disabled:
+	// Feed title
 	hbox[i] = gtk_hbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox[i]), 15);
 
@@ -294,7 +299,7 @@ void quick_setup(Params *prm0)
 	gtk_assistant_set_page_title(GTK_ASSISTANT(qsetup), page[i], "Display feed title");
 	i++;
 
-	/* Item title */
+	// Item title
 	hbox[i] = gtk_hbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox[i]), 15);
 
@@ -322,7 +327,7 @@ void quick_setup(Params *prm0)
 	gtk_assistant_set_page_title(GTK_ASSISTANT(qsetup), page[i], "Display item title");
 	i++;
 
-	/* Item description */
+	// Item description
 	hbox[i] = gtk_hbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox[i]), 15);
 
@@ -349,17 +354,14 @@ void quick_setup(Params *prm0)
 	gtk_assistant_set_page_type(GTK_ASSISTANT(qsetup), page[i], GTK_ASSISTANT_PAGE_CONTENT);
 	gtk_assistant_set_page_title(GTK_ASSISTANT(qsetup), page[i], "Display item description");
 	i++;
+*/
 
 	/* "Open in browser" command line */
 	hbox[i] = gtk_hbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox[i]), 15);
 
-#ifndef G_OS_WIN32
-	label[i] = gtk_label_new("'Open in Browser' command line:");
-#else
-	label[i] = gtk_label_new("'Open in Browser' command line:\n"
-		"<small>(leave blank if unsure)</small>");
-#endif
+	label[i] = gtk_label_new("Browser to open links with:\n"
+		"<small>(Shell command - leave blank/unchanged if unsure)</small>");
 	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
 	gtk_box_pack_start(GTK_BOX(hbox[i]), label[i], FALSE, FALSE, 0);
@@ -381,7 +383,8 @@ void quick_setup(Params *prm0)
 	gtk_assistant_set_page_title(GTK_ASSISTANT(qsetup), page[i], "Pick a Browser");
 	i++;
 
-	/* Disable popups */
+/* This page also disabled:
+	// Disable popups
 	hbox[i] = gtk_hbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox[i]), 15);
 
@@ -408,13 +411,15 @@ void quick_setup(Params *prm0)
 	gtk_assistant_set_page_type(GTK_ASSISTANT(qsetup), page[i], GTK_ASSISTANT_PAGE_CONTENT);
 	gtk_assistant_set_page_title(GTK_ASSISTANT(qsetup), page[i], "Disable popups");
 	i++;
+*/
 
 	/* Confirm page */
 	hbox[i] = gtk_hbox_new(FALSE, 0);
 	gtk_container_set_border_width(GTK_CONTAINER(hbox[i]), 15);
 
 	label[i] = gtk_label_new("Quick setup complete !\n"
-		"To open the full setup window, right-click then 'Edit > Preferences'.");
+		"To open the preferences or full settings window, right-click on " APP_NAME " then\n"
+		" 'Edit > Preferences' or 'Edit > Full Settings'.");
 	gtk_label_set_use_markup(GTK_LABEL(label[i]), TRUE);
 	gtk_misc_set_alignment(GTK_MISC(label[i]), 0, 0.5);
 	gtk_box_pack_start(GTK_BOX(hbox[i]), label[i], FALSE, FALSE, 0);
@@ -432,11 +437,11 @@ void quick_setup(Params *prm0)
 
 	g_signal_connect(G_OBJECT(spinbut_winy), "changed", G_CALLBACK(keep_setting), (gpointer)1);
 	g_signal_connect(G_OBJECT(checkbut_alwaysontop), "toggled", G_CALLBACK(keep_setting), (gpointer)2);
-	g_signal_connect(G_OBJECT(checkbut_feedtitle), "toggled", G_CALLBACK(keep_setting), (gpointer)3);
+	/*g_signal_connect(G_OBJECT(checkbut_feedtitle), "toggled", G_CALLBACK(keep_setting), (gpointer)3);
 	g_signal_connect(G_OBJECT(checkbut_itemtitle), "toggled", G_CALLBACK(keep_setting), (gpointer)4);
-	g_signal_connect(G_OBJECT(checkbut_itemdes), "toggled", G_CALLBACK(keep_setting), (gpointer)5);
-	g_signal_connect(G_OBJECT(entry_openlinkcmd), "changed", G_CALLBACK(keep_setting), (gpointer)6);
-	g_signal_connect(G_OBJECT(checkbut_nopopups), "toggled", G_CALLBACK(keep_setting), (gpointer)7);
+	g_signal_connect(G_OBJECT(checkbut_itemdes), "toggled", G_CALLBACK(keep_setting), (gpointer)5);*/
+	g_signal_connect(G_OBJECT(entry_openlinkcmd), "changed", G_CALLBACK(keep_setting), (gpointer)3/*6*/);
+	/*g_signal_connect(G_OBJECT(checkbut_nopopups), "toggled", G_CALLBACK(keep_setting), (gpointer)7);*/
 
 	g_signal_connect(G_OBJECT(qsetup), "cancel", G_CALLBACK(qsetup_cancel), NULL);
 	g_signal_connect(G_OBJECT(qsetup), "prepare", G_CALLBACK(qsetup_prepare), NULL);
