@@ -1,4 +1,5 @@
-/*	Very basic xlib prog
+/*
+ *	Very basic xlib prog
  *	Compile with: gcc -o xlib-prog xlib-prog.c -lX11
  */
 
@@ -28,12 +29,15 @@ int main(int argc, char *argv[])
 		displ_w = DisplayWidth(displ, screen_num);
 		displ_h = DisplayHeight(displ, screen_num);
 
-		win_w = displ_w / 2;
-		win_h = displ_h / 2;
 		win_x = displ_w / 4;
 		win_y = displ_h / 4;
+		win_w = displ_w / 2;
+		win_h = displ_h / 2;
 		win_border_w = 2;
 
+		fprintf(stdout, "win_x = %d, win_y = %d, win_w = %d, win_h = %d\n",
+			win_x, win_y, win_w, win_h);
+
 		win = XCreateSimpleWindow(
 			displ, RootWindow(displ, screen_num),
 			win_x, win_y, win_w, win_h, win_border_w,
@@ -43,6 +47,10 @@ int main(int argc, char *argv[])
 		XMapWindow(displ, win);
 		XSync(displ, False);
 
+		/* Need this to get the correct win position */
+		XMoveWindow(displ, win, win_x, win_y);
+		XSync(displ, False);
+
 		sleep(5);
 		XCloseDisplay(displ);
 		return 0;
